@@ -1,4 +1,4 @@
-# (c) 2015-2018 Acellera Ltd http://www.acellera.com
+# (c) 2015-2022 Acellera Ltd http://www.acellera.com
 # All Rights Reserved
 # Distributed under HTMD Software License Agreement
 # No redistribution in whole or part
@@ -11,7 +11,7 @@ import logging
 logger = logging.getLogger(__name__)
 
 
-chain_alphabet = list(string.digits + string.ascii_uppercase + string.ascii_lowercase)
+chain_alphabet = list(string.ascii_uppercase + string.ascii_lowercase + string.digits)
 segid_alphabet = list(string.ascii_uppercase + string.ascii_lowercase + string.digits)
 
 
