@@ -1,4 +1,4 @@
-# (c) 2015-2018 Acellera Ltd http://www.acellera.com
+# (c) 2015-2022 Acellera Ltd http://www.acellera.com
 # All Rights Reserved
 # Distributed under HTMD Software License Agreement
 # No redistribution in whole or part
@@ -10,6 +10,8 @@ import numpy as np
 import pandas as pd
 import os
 from moleculekit.molecule import Molecule, mol_equal, UniqueResidueID
+from moleculekit.tools.autosegment import autoSegment2
+from moleculekit.util import sequenceID
 
 
 logger = logging.getLogger(__name__)
@@ -31,7 +33,6 @@ def _warn_if_contains_DUM(mol):
 
 
 def _check_chain_and_segid(mol, verbose):
-    from moleculekit.util import sequenceID
     import string
 
     emptychains = mol.chain == ""
@@ -75,26 +76,18 @@ def _check_chain_and_segid(mol, verbose):
             lastatom = chainatoms[-1]
             print(f"Chain {c}:")
             print(
-                f"    First residue: {mol.resname[firstatom]}:{mol.resid[firstatom]}:{mol.insertion[firstatom]}"
+                f"    First residue: {_fmt_res(mol.resname[firstatom], mol.resid[firstatom], mol.insertion[firstatom])}"
             )
             print(
-                f"    Final residue: {mol.resname[lastatom]}:{mol.resid[lastatom]}:{mol.insertion[lastatom]}"
+                f"    Final residue: {_fmt_res(mol.resname[lastatom], mol.resid[lastatom], mol.insertion[lastatom])}"
             )
         print("---- End of chain report ----\n")
 
     return mol
 
 
-def _gen_resname():
-    import string
-
-    for lett in string.ascii_uppercase:
-        for num in range(100):
-            yield f"{lett}{num:02d}"
-
-
 def _generate_nonstandard_residues_ff(
-    mol, definition, forcefield, _molkit_ff=True, outdir=None
+    mol, definition, forcefield, _molkit_ff=True, outdir=None, ignore_ns_errors=False
 ):
     import tempfile
     from moleculekit.tools.preparation_customres import _get_custom_ff
@@ -104,8 +97,6 @@ def _generate_nonstandard_residues_ff(
         _mol_to_xml_def,
     )
 
-    _resname_gen = _gen_resname()
-
     uqprot_resn = np.unique(mol.get("resname", sel="protein"))
     not_in_ff = []
     for resn in uqprot_resn:
@@ -116,52 +107,44 @@ def _generate_nonstandard_residues_ff(
         return definition, forcefield
 
     try:
-        import aceprep
+        from aceprep.prepare import RDKprepare
     except ImportError:
+        if ignore_ns_errors:
+            return definition, forcefield
         raise RuntimeError(
-            "To protonate non-standard aminoacids you need the aceprep library. Please contact Acellera info@acellera.com for more information."
+            "To protonate non-canonical aminoacids you need the aceprep library. Please contact Acellera info@acellera.com for more information or set ignore_ns_errors=True to ignore non-canonical residues in the protonation (this will leave the residues unprotonated)."
         )
 
     with tempfile.TemporaryDirectory() as tmpdir:
         for res in not_in_ff:
             try:
-                logger.warning(f"Attempting to template non-standard residue {res}...")
+                logger.info(f"Attempting to template non-canonical residue {res}...")
+                # This removes the non-canonical hydrogens from the original mol object
+                mol.remove((mol.resname == res) & (mol.element == "H"), _logger=False)
                 molc = mol.copy()
+
                 # Hacky way of getting the first molecule, if there are copies
                 molresn = molc.resname == res
-                firstname = mol.name[molresn][0]
-                lastname = mol.name[molresn][-1]
-                start = np.where(molresn & (mol.name == firstname))[0][0]
-                end = np.where(molresn & (mol.name == lastname))[0][0]
+                firstname = molc.name[molresn][0]
+                lastname = molc.name[molresn][-1]
+                start = np.where(molresn & (molc.name == firstname))[0][0]
+                end = np.where(molresn & (molc.name == lastname))[0][0]
                 # Remove all other stuff
                 molc.filter(f"index {start} to {end}", _logger=False)
                 cres = _generate_custom_residue(molc, res)
 
-                # Rename custom residue if it's integer!
-                try:
-                    int(res)
-                except Exception:
-                    pass
-                else:
-                    new_rn = next(_resname_gen)
-                    while new_rn in list(resn):
-                        new_rn = next(_resname_gen)
-                    logger.warning(
-                        f"Integer residue name {res} was renamed to {new_rn} to avoid preparation issues."
-                    )
-                    mol.resname[mol.resname == res] = new_rn
-                    cres.resname[:] = new_rn
-                    res = new_rn
-
                 _mol_to_xml_def(cres, os.path.join(tmpdir, f"{res}.xml"))
                 _mol_to_dat_def(cres, os.path.join(tmpdir, f"{res}.dat"))
                 if outdir is not None:
                     os.makedirs(outdir, exist_ok=True)
                     cres.write(os.path.join(outdir, f"{res}.cif"))
-                logger.warning(f"Succesfully templated non-standard residue {res}.")
+                logger.info(f"Succesfully templated non-canonical residue {res}.")
             except Exception as e:
+                import traceback
+
+                traceback.print_exc()
                 raise RuntimeError(
-                    f"Failed to protonate non-standard residue {res}. Please remove it from the protein or mutate it to continue preparation. Detailed error message: {e}"
+                    f"Failed to protonate non-canonical residue {res}. Please remove it from the protein or mutate it to continue preparation. Detailed error message: {e}"
                 )
         definition, forcefield = _get_custom_ff(user_ff=tmpdir, molkit_ff=_molkit_ff)
     return definition, forcefield
@@ -170,7 +153,7 @@ def _generate_nonstandard_residues_ff(
 def _detect_nonpeptidic_bonds(mol):
     coordination_ions = ("Ca", "Zn", "Mn", "Cu", "Fe", "Ni", "Mo", "Re", "Na")
 
-    prot_idx = mol.atomselect("protein", indexes=True)
+    prot_idx = mol.atomselect("protein or resname ACE NME", indexes=True)
     ion_idx = np.where(np.isin(mol.element, coordination_ions))[0]
     same_resname = np.isin(
         mol.resname[ion_idx], [cc.upper() for cc in coordination_ions]
@@ -236,7 +219,6 @@ def _pdb2pqr(
     no_prot=None,
     no_opt=None,
     no_titr=None,
-    force_protonation=None,
     propka_args=None,
 ):
     from pdb2pqr.io import get_molecule
@@ -273,7 +255,7 @@ def _pdb2pqr(
 
     hydrogen_handler = hydrogens.create_handler()
     debumper = Debump(biomolecule)
-    pkas = None
+    pka_list = None
     if assign_only:
         # TODO - I don't understand why HIS needs to be set to HIP for
         # assign-only
@@ -292,20 +274,21 @@ def _pdb2pqr(
         if titrate:
             biomolecule.remove_hydrogens()
             pka_list, _ = run_propka(propka_args, biomolecule)
-            # logger.info(f"PROPKA information:\n{pka_str}")
+            # Remove terminal pkas
+            pka_list = [
+                row
+                for row in pka_list
+                if row["group_label"].startswith(row["res_name"])
+            ]
 
             # STEFAN mod: Delete pka values for residues we should not titrate
             pkas = _delete_no_titrate(pka_list, no_titr)
-            # STEFAN mod: Force specific protonation states to residues
-            _apply_patches_force_prot(biomolecule, force_protonation)
-
             biomolecule.apply_pka_values(
                 forcefield_.name,
                 ph,
                 {
                     f"{row['res_name']} {row['res_num']} {row['chain_id']}": row["pKa"]
                     for row in pkas
-                    if row["group_label"].startswith(row["res_name"])
                 },
             )
 
@@ -358,7 +341,7 @@ def _pdb2pqr(
     name_scheme = forcefield.Forcefield("amber", definition, None)
     biomolecule.apply_name_scheme(name_scheme)
 
-    return missing_atoms, pkas, biomolecule
+    return missing_atoms, pka_list, biomolecule
 
 
 def _atomsel_to_hold(mol_in, sel):
@@ -423,7 +406,7 @@ def _get_hold_residues(
                 # Only disable protonation if there is no force-protonation
                 _no_prot += val
 
-    return _no_opt, _no_prot, _no_titr, _force_prot
+    return _no_opt, _no_prot, _no_titr
 
 
 def _check_frozen_histidines(mol_in, _no_prot):
@@ -446,17 +429,20 @@ def _check_frozen_histidines(mol_in, _no_prot):
 
 
 def _prepare_nucleics(mol):
-    # Renames residues to the names expected by PARSE. Fixes issues with 5' G atoms.
+    # Renames residues to the names expected by PARSE. Fixes issues with 5' atoms.
     nucl_mask = mol.atomselect("nucleic")
     uq_resn = mol.resname[nucl_mask]
 
-    # Clean 5' G problematic atoms
+    # Clean 5' dangling P atom
     for ch in np.unique(mol.get("chain", "nucleic")):
+        # First residue of that chain
         resid = mol.resid[(mol.chain == ch) & mol.atomselect("nucleic")][0]
-        mol.remove(
-            f"chain {ch} and resid {resid} and (resname G) and (name P OP1 OP2)",
-            _logger=False,
+        remsel = (
+            (mol.chain == ch)
+            & (mol.resid == resid)
+            & np.isin(mol.name, ("P", "OP1", "OP2"))
         )
+        mol.remove(remsel, _logger=False)
 
     mapping = {
         "T": "DT",
@@ -473,6 +459,25 @@ def _prepare_nucleics(mol):
         mol.resname[mol.resname == res] = mapping[res]
 
 
+def _fix_protonation_resnames(mol):
+    # This function patches the names of residues to their correct protonation state if they have specific hydrogens
+    uqres = sequenceID((mol.resid, mol.insertion, mol.chain))
+    for uq in uqres:
+        resatm = uqres == uq
+        resname = mol.resname[resatm][0]
+        names = mol.name[resatm]
+        if (resname == "GLU") and ("HE2" in names):
+            mol.resname[resatm] = "GLH"
+        elif (resname == "ASP") and ("HD2" in names):
+            mol.resname[resatm] = "ASH"
+        elif (resname == "HIS") and all(np.isin(("HD1", "HD2", "HE1", "HE2"), names)):
+            mol.resname[resatm] = "HIP"
+        elif (resname == "HIS") and all(np.isin(("HD1", "HD2", "HE1"), names)):
+            mol.resname[resatm] = "HID"
+        elif (resname == "HIS") and all(np.isin(("HD2", "HE1", "HE2"), names)):
+            mol.resname[resatm] = "HIE"
+
+
 def proteinPrepare(
     mol_in,
     pH=7.0,
@@ -509,6 +514,7 @@ def systemPrepare(
     return_details=False,
     hydrophobic_thickness=None,
     plot_pka=None,
+    ignore_ns_errors=False,
     _logger_level="ERROR",
     _molkit_ff=True,
     outdir=None,
@@ -604,6 +610,9 @@ def systemPrepare(
         ignoring the covalent bond, meaning it may break the bonds or add hydrogen atoms between the bonds.
     plot_pka : str
         Provide a file path with .png extension to draw the titration diagram for the system residues.
+    ignore_ns_errors : bool
+        If False systemPrepare will issue an error when it fails to protonate non-canonical residues in the protein.
+        If True it will ignore errors on non-canonical residues leaving them unprotonated.
     outdir : str
         A path where to save custom residue cif files used for building
 
@@ -677,19 +686,33 @@ def systemPrepare(
     _warn_if_contains_DUM(mol_in)
 
     mol_in = _check_chain_and_segid(mol_in, verbose)
+    mol_orig = mol_in.copy()
 
     _prepare_nucleics(mol_in)
+    _fix_protonation_resnames(mol_in)
 
     definition, forcefield = _get_custom_ff(molkit_ff=_molkit_ff)
     definition, forcefield = _generate_nonstandard_residues_ff(
-        mol_in, definition, forcefield, _molkit_ff, outdir
+        mol_in,
+        definition,
+        forcefield,
+        _molkit_ff,
+        outdir,
+        ignore_ns_errors=ignore_ns_errors,
     )
 
     nonpept = None
     if hold_nonpeptidic_bonds:
         nonpept = _detect_nonpeptidic_bonds(mol_in)
 
-    _no_opt, _no_prot, _no_titr, _force_prot = _get_hold_residues(
+    if force_protonation is not None:
+        for sel, resn in force_protonation:
+            # Remove side-chain hydrogens
+            mol_in.remove(f"({sel}) and not backbone and element H", _logger=False)
+            # Rename to desired protonation state
+            mol_in.set("resname", resn, sel=sel)
+
+    _no_opt, _no_prot, _no_titr = _get_hold_residues(
         mol_in, no_opt, no_prot, no_titr, force_protonation, nonpept
     )
     _check_frozen_histidines(mol_in, _no_prot)
@@ -705,7 +728,6 @@ def systemPrepare(
             no_opt=_no_opt,
             no_prot=_no_prot,
             no_titr=_no_titr,
-            force_protonation=_force_prot,
             definition=definition,
             forcefield_=forcefield,
         )
@@ -721,13 +743,16 @@ def systemPrepare(
     mol_out.box = mol_in.box
     _fixup_water_names(mol_out)
 
-    df = _create_table(mol_in, mol_out, pka_df)
+    df = _create_table(mol_orig, mol_out, pka_df)
 
     _list_modifications(df)
 
     if titration:
         if plot_pka is not None:
-            _get_pka_plot(df, plot_pka)
+            try:
+                _get_pka_plot(df, plot_pka)
+            except Exception as e:
+                logger.error(f"Failed at generating pKa plot with error {e}")
         _warn_pk_close_to_ph(df, pH)
         if hydrophobic_thickness:
             # TODO: I think this only works if the protein is assumed aligned to the membrane and the membrane is centered at Z=0
@@ -814,9 +839,10 @@ def _create_table(mol_in, mol_out, pka_df):
             & (mol_out.chain == tup[3])
             & (mol_out.segid == tup[4])
         )
+
         if not np.any(new_mask):
             raise RuntimeError(
-                f"Unable to find residue {' '.join(tup)} after preparation"
+                f"Unable to find residue {' '.join(map(str, tup))} after preparation"
             )
         new_idx = np.where(new_mask)[0][0]
         resname = mol_out.resname[new_idx]
@@ -859,6 +885,10 @@ def _create_table(mol_in, mol_out, pka_df):
     return df
 
 
+def _fmt_res(resname, resid, insertion, chain=""):
+    return f"{resname:<4s} {resid:>4d}{insertion.strip():<s}{chain:>2s}"
+
+
 def _list_modifications(df):
     for _, row in df[df.resname != df.protonation].iterrows():
         if row.resname in ["HOH", "WAT"]:
@@ -869,7 +899,9 @@ def _list_modifications(df):
         ch = row.chain
         rid = row.resid
         ins = row.insertion.strip()
-        logger.info(f"Modified residue {old_resn}:{ch}:{rid}{ins} to {new_resn}")
+        logger.info(
+            f"Modified residue {_fmt_res(old_resn, rid, ins, ch)} to {new_resn}"
+        )
 
 
 def _warn_pk_close_to_ph(df, pH, tol=1.0):
@@ -880,7 +912,7 @@ def _warn_pk_close_to_ph(df, pH, tol=1.0):
         )
         for _, dr in dubious.iterrows():
             logger.warning(
-                f"Dubious protonation state:    {dr.resname:4s} {dr.resid:>4d}{dr.insertion.strip()} {dr.chain:1s} (pKa={dr.pKa:5.2f})"
+                f"Dubious protonation state:    {_fmt_res(dr.resname, dr.resid, dr.insertion, dr.chain)} (pKa={dr.pKa:5.2f})"
             )
 
 
@@ -912,6 +944,12 @@ def _get_pka_plot(df, outname, pH=7.4, figSizeX=13, dpk=1.0, font_size=12):
     from matplotlib.colors import LinearSegmentedColormap
     import matplotlib.patheffects as PathEffects
 
+    if not np.any(df.pKa.notnull()):
+        logger.warning(
+            "No pka plot was generated due to no pKa values calculated by propka."
+        )
+        return
+
     # Shading
     Xe = np.array([[1, 0], [1, 0]])
 
@@ -1101,7 +1139,7 @@ def _get_pka_plot(df, outname, pH=7.4, figSizeX=13, dpk=1.0, font_size=12):
 
 # The below is used for testing only
 try:
-    import aceprep
+    from aceprep.prepare import RDKprepare
 except ImportError:
     ACEPREP_EXISTS = False
 else:
@@ -1116,6 +1154,13 @@ class _TestPreparation(unittest.TestCase):
         self.home = home(dataDir="test-systemprepare")
 
     def _compare_results(self, refpdb, refdf_f, pmol, df: pd.DataFrame):
+        from moleculekit.util import tempname
+
+        # # Use this to update tests
+        # pmol.filter("not water", _logger=False)
+        # pmol.write(refpdb, writebonds=False)
+        # df.to_csv(refdf_f, index=False)
+
         refdf = pd.read_csv(
             refdf_f, dtype=_table_dtypes, keep_default_na=False, na_values=[""]
         )
@@ -1128,39 +1173,45 @@ class _TestPreparation(unittest.TestCase):
                 raise AssertionError(
                     "Different number of residues were found in the DataFrames!"
                 )
-            for key in [
-                "resname",
-                "protonation",
-                "resid",
-                "insertion",
-                "chain",
-                "segid",
-                "pKa",
-                "buried",
-            ]:
-                refv = refdf[key].values
-                newv = df[key].values
-                diff = refv != newv
-                if key in ("pKa", "buried"):
-                    nans = refv == ""
-                    refv = refv[~nans].astype(np.float32)
-                    newv = newv[~nans].astype(np.float32)
-                    diff = np.abs(refv - newv) > 1e-4
-                if any(diff):
-                    print(f"Found difference in field: {key}")
-                    print(f"Reference values: {refv[diff]}")
-                    print(f"New values:       {newv[diff]}")
-            raise
+            try:
+                for key in [
+                    "resname",
+                    "protonation",
+                    "resid",
+                    "insertion",
+                    "chain",
+                    "segid",
+                    "pKa",
+                    "buried",
+                ]:
+                    refv = refdf[key].values
+                    newv = df[key].values
+                    diff = refv != newv
+                    if key in ("pKa", "buried"):
+                        nans = refv == ""
+                        refv = refv[~nans].astype(np.float32)
+                        newv = newv[~nans].astype(np.float32)
+                        diff = np.abs(refv - newv) > 1e-4
+                    if any(diff):
+                        print(f"Found difference in field: {key}")
+                        print(f"Reference values: {refv[diff]}")
+                        print(f"New values:       {newv[diff]}")
+            except Exception:
+                pass
+
+            df_out = tempname(suffix=".csv")
+            df.to_csv(df_out, index=False)
+            raise AssertionError(f"Failed comparison of {refdf_f} to {df_out}")
 
         refmol = Molecule(refpdb)
-        refmol.filter("not water")
-        pmol.filter("not water")
+        refmol.filter("not water", _logger=False)
+        pmol.filter("not water", _logger=False)
         assert mol_equal(
             refmol, pmol, exceptFields=["serial"], fieldPrecision={"coords": 1e-3}
         )
 
     def test_systemPrepare(self):
-        pdbids = ["3PTB", "1A25", "1U5U"]
+        pdbids = ["3PTB", "1A25", "1U5U", "1UNC", "6A5J"]
         for pdb in pdbids:
             with self.subTest(pdbid=pdb):
                 test_home = os.path.join(self.home, pdb)
@@ -1252,47 +1303,46 @@ class _TestPreparation(unittest.TestCase):
             df,
         )
 
+    @unittest.skipUnless(ACEPREP_EXISTS, "Can only run with aceprep installed")
     def test_nonstandard_residues(self):
         test_home = os.path.join(self.home, "test-nonstandard-residues")
-        mol = Molecule(os.path.join(test_home, "1A4W.pdb"))
+        files = {
+            "1A4W.pdb": "1A4W_prepared",
+            "5VBL.pdb": "5VBL_prepared",
+            "2QRV.pdb": "2QRV_prepared",
+        }
+        for inf, outf in files.items():
+            with self.subTest(file=inf):
+                mol = Molecule(os.path.join(test_home, inf))
+                if inf == "2QRV.pdb":
+                    mol = autoSegment2(mol, fields=("chain", "segid"))
 
-        pmol, df = systemPrepare(mol, return_details=True, hold_nonpeptidic_bonds=True)
+                pmol, df = systemPrepare(
+                    mol, return_details=True, hold_nonpeptidic_bonds=True
+                )
 
-        self._compare_results(
-            os.path.join(test_home, "1A4W_prepared.pdb"),
-            os.path.join(test_home, "1A4W_prepared.csv"),
-            pmol,
-            df,
-        )
+                self._compare_results(
+                    os.path.join(test_home, f"{outf}.pdb"),
+                    os.path.join(test_home, f"{outf}.csv"),
+                    pmol,
+                    df,
+                )
 
-    @unittest.skipUnless(ACEPREP_EXISTS, "Can only run with aceprep installed")
-    def test_nonstandard_residue_hard(self):
+    @unittest.skipIf(ACEPREP_EXISTS, "Can only run WITHOUT aceprep installed")
+    def test_nonstandard_residue_hard_ignore_ns(self):
         test_home = os.path.join(self.home, "test-nonstandard-residues")
         mol = Molecule(os.path.join(test_home, "5VBL.pdb"))
 
         pmol, df = systemPrepare(
-            mol, return_details=True, hold_nonpeptidic_bonds=True, _molkit_ff=False
+            mol,
+            return_details=True,
+            hold_nonpeptidic_bonds=True,
+            _molkit_ff=False,
+            ignore_ns_errors=True,
         )
-
         self._compare_results(
-            os.path.join(test_home, "5VBL_prepared.pdb"),
-            os.path.join(test_home, "5VBL_prepared.csv"),
-            pmol,
-            df,
-        )
-
-    @unittest.skipUnless(ACEPREP_EXISTS, "Can only run with aceprep installed")
-    def test_unknown_nonstandard_residues(self):
-        test_home = os.path.join(self.home, "test-nonstandard-residues")
-        mol = Molecule(os.path.join(test_home, "1A4W.pdb"))
-
-        pmol, df = systemPrepare(
-            mol, return_details=True, hold_nonpeptidic_bonds=True, _molkit_ff=False
-        )
-
-        self._compare_results(
-            os.path.join(test_home, "1A4W_prepared.pdb"),
-            os.path.join(test_home, "1A4W_prepared.csv"),
+            os.path.join(test_home, "5VBL_prepared_ignore_ns.pdb"),
+            os.path.join(test_home, "5VBL_prepared_ignore_ns.csv"),
             pmol,
             df,
         )
