@@ -1,4 +1,4 @@
-# (c) 2015-2018 Acellera Ltd http://www.acellera.com
+# (c) 2015-2022 Acellera Ltd http://www.acellera.com
 # All Rights Reserved
 # Distributed under HTMD Software License Agreement
 # No redistribution in whole or part
@@ -119,7 +119,10 @@ def sequenceStructureAlignment(
     segment_type = segment_type_mol
 
     if segment_type == "protein":
-        alignments = pairwise2.align.globaldx(seqref, seqmol, matlist.blosum62)
+        # -11 is gap creation penalty. -1 is gap extension penalty. Taken from https://www.arabidopsis.org/Blast/BLASToptions.jsp BLASTP options
+        alignments = pairwise2.align.globalds(
+            seqref, seqmol, matlist.blosum62, -11.0, -1.0
+        )
     elif segment_type == "nucleic":
         alignments = pairwise2.align.globalxx(seqref, seqmol)
     numaln = len(alignments)
