@@ -1,4 +1,4 @@
-# (c) 2015-2018 Acellera Ltd http://www.acellera.com
+# (c) 2015-2022 Acellera Ltd http://www.acellera.com
 # All Rights Reserved
 # Distributed under HTMD Software License Agreement
 # No redistribution in whole or part
@@ -526,7 +526,7 @@ def opm(pdb, keep=False, keepaltloc="A"):
     True
 
     """
-    import urllib
+    import urllib.request
     import re
     from moleculekit.support import string_to_tempfile
     from moleculekit.molecule import Molecule
